
#ifndef BOOST_MPL_LIMITS_MAP_HPP_INCLUDED
#define BOOST_MPL_LIMITS_MAP_HPP_INCLUDED

// Copyright Aleksey Gurtovoy 2000-2004
//
// Distributed under the Boost Software License, Version 1.0. 
// (See accompanying file LICENSE_1_0.txt or copy at 
// http://www.boost.org/LICENSE_1_0.txt)
//
// See http://www.boost.org/libs/mpl for documentation.

// $Id: map.hpp,v 1.1 2009/08/23 12:39:45 pschaefer Exp $
// $Date: 2009/08/23 12:39:45 $
// $Revision: 1.1 $

#if !defined(BOOST_MPL_LIMIT_MAP_SIZE)
#   define BOOST_MPL_LIMIT_MAP_SIZE 20
#endif

#endif // BOOST_MPL_LIMITS_MAP_HPP_INCLUDED
